#!/bin/sh
#
# This is a driver installer for IRTOUCH Touch Screen.
#
# Company: Beijing IRTOUCH SYSTEMS Co.,Ltd
# Version: 1.0.0
# Released: 12/13/2010
# Support: zh@unitop.cc
#
  
version="1.0.0"
driverName="irtouch.bin.tar.gz"
companyName="irtouch"
installPath="/usr/local"

xorgPath="/etc/X11/xorg.conf"
xConfigPath="/etc/X11/XF86Config-4"
xConfigPath2="/etc/X11/XF86Config"

xModulesPath="/usr/X11R6/lib/modules/input"
xModulesPath2="/usr/lib/xorg/modules/input"

blackListPath="/etc/modprobe.d/blacklist"
blackListPath1="/etc/modprobe.d/blacklist.conf"
blackListPath2="/etc/modprobe.d/blacklist-compat" # for Mandriva 2008
blackListPath3="/etc/hotplug/blacklist"

localPath="/etc/rc.d/rc.local"
localPath1="/etc/rc.local"
localPath2="/etc/rc.d/boot.local"
localPath3="/etc/boot.local"

trash="/dev/null"
servicePath="/usr/local/irtouch/irtouch.init"
tsConfigName="irtouch/irtouch.conf"
saveConfigPath="/etc"

ShowTitle() {
    echo ""
    echo "#######################################################"
    echo "#                                                     #"
    echo "#          Beijing IRTOUCH SYSTEMS Co.,Ltd            #"
    echo "#                                                     #"
    echo "#         Linux driver installer for IRTOUCH          #"
    echo "#                                                     #"
    echo "#######################################################"
    echo ""
}

CheckPermission() {
    echo -n "(I) Check user permission:"
    account=`whoami`
    if [ ${account} = "root" ]; then
        echo " ${account}, you are the supervisor."
 	echo ""
    else
        echo " ${account}, you are NOT the supervisor."
        echo "(E) The root permission is required to run this installer."
        echo ""
        exit 1
    fi
}

CheckOldDriver() {
    temppath=`find ${installPath} -name ${companyName}`
    olddriverpath=${temppath%/*}
    if [ -z "${olddriverpath}" ]; then
        echo "(I) Begin to setup the IRTOUCH driver."
    else
        echo "(W) Found the old IRTOUCH driver."
        echo "(E) It has to be removed before run this installer."
    fi
}

ExtractDriver() {
    if [ -e ${installPath}/${driverName} ]; then
        rm -f ${installPath}/${driverName}
    fi
    
    if [ -e ${installPath}/${companyName}} ]; then
        rm -rf ${installPath}/${companyName}
        echo "(I) Found and removed previous IRTOUCH driver."
    fi

    cp -f ${driverName} ${installPath}
    ( cd ${installPath}; tar -zxf ${driverName}; rm -f ${driverName} )
    temppath=`find ${installPath} -name ${companyName}`
    driverpath=${temppath%/*}
    echo "(I) Extract IRTOUCH driver archive to ${driverpath}."

    cp -af ${installPath}/${companyName}/${companyName} ${saveConfigPath}
    echo "(I) Copy configuration file to /etc."

    ln -sf ${installPath}/${companyName}/calib /usr/bin
    echo "(I) Create calib link in /usr/bin."
}

RemoveDriver() {
    if [ -e ${installPath}/${driverName} ]; then	
        rm -rf ${installPath}/${driverName}
    fi
	
    temppath=`find ${installPath} -name ${companyName}`
    driverpath=${temppath%/*}
    if [ -z "${driverpath}" ]; then
        echo "(E) The driver archive has been removed already."
        echo ""
        exit 1
    elif [ -e ${installPath}/${companyName} ]; then
	${installPath}/${companyName}/irtouch.init stop
	rm -rf ${installPath}/${companyName}
	echo "(I) Removed IRTOUCH driver archive from ${installPath}/${companyName}."
    fi
}

CheckXorgVersion() {
    checkx="X -version"
    xfile="X.info"
    ${checkx} > ${xfile} 2>&1
    grep -q "X.Org X Server 1.4." ${xfile}
    if [ $? -eq 0 ]; then
        grep -q "X.Org X Server 1.4.99" ${xfile}
        if [ $? -eq 0 ]; then
            echo "(I) Check X window version: 1.4.99.x" # Fedora 9 : 1.4.99.901
            xdirectory="x1499"
        else
            echo "(I) Check X window version: 1.4.x" # openSuSE 11 / Ubuntu 8.04: 1.4.0.90
            xdirectory="x14"
        fi
    else
        grep -q "X Window System Version 1.3." ${xfile} # Fedora 8 / Ubuntu 7.10: 1.3.0
        if [ $? -eq 0 ]; then
            echo "(I) Check X window version: 1.3.x"
            xdirectory="x13"
        else
            grep -q "X.Org X Server 1.5." ${xfile} # Ubuntu 8.10 / Fedora 10
            if [ $? -eq 0 ]; then
                echo "(I) Check X window version: 1.5.x"
                xdirectory="x15"
            else
                grep -q "X.Org X Server 1.6." ${xfile} # Ubuntu 9.04 / Ubuntu 9.10 / Fedora 11
                if [ $? -eq 0 ]; then
                    echo "(I) Check X window version: 1.6.x"
                    xdirectory="x16"
                else
                    grep -q "X.Org X Server 1.7." ${xfile} # Fedora 12 / Ubuntu 10.04
                    if [ $? -eq 0 ]; then
                        echo "(I) Check X window version: 1.7.x"
                        xdirectory="x17"
                    else
 			grep -q "X.Org X Server 1.8." ${xfile} # Fedora 13
                    	if [ $? -eq 0 ]; then
                            echo "(I) Check X window version: 1.8.x"
                            xdirectory="x18"
                        else
			    grep -q "X.Org X Server 1.9." ${xfile} # Fedora 14
		            if [ $? -eq 0 ]; then
		                 echo "(I) Check X window version: 1.9.x"
		                 xdirectory="x19"
		            else
				grep -q "X.Org X Server 1.10." ${xfile} # Ubuntu 11.04
			        if [ $? -eq 0 ]; then
			             echo "(I) Check X window version: 1.10.x"
			             xdirectory="x110"
			        else
				    grep -q "X.Org X Server 1.11." ${xfile} # Fedora Core 16
				    if [ $? -eq 0 ]; then
                                       	echo "(I) Check X window version: 1.11.x"
                                       	xdirectory="x111"
				    else
					grep -q "X.Org X Server 1.12." ${xfile} # Fedora Core 17
					if [ $? -eq 0 ]; then
					    echo "(I) Check X window version: 1.12.x"
					    xdirectory="x112"
					else
                                       	     grep -q "X Window System Version 6.8.99.900" ${xfile} # Mandriva 2006
                                             if [ $? -eq 0 ]; then
                                                 echo "(I) Check X window version: 6.8.99.900"
                                                 xmodule="irtouch_drv.o"
                                                 xdirectory="x67"
                                             else
                                                 echo "(I) Check X window version: 6.9.0 ~ 7.2.0"
                                                 xdirectory="x69"
					     fi
					 fi
				     fi
				 fi
			     fi
			 fi
                    fi
                fi
            fi
        fi
    fi
    rm -f ${xfile}
}

CopyXorgMudule() {
    if [ -e ${xModulesPath}/vmmouse_drv.*o ]; then
        mousepath=`find ${xModulesPath} -name vmmouse_drv.*o`
    elif [ -e ${xModulesPath2}/vmmouse_drv.*o ]; then
        mousepath=`find ${xModulesPath2} -name vmmouse_drv.*o`
    else
        echo "(I) Searching the X input modules directory."
        mousepath=`find /usr -name vmmouse_drv.*o`
        if [ -z "${mousepath}" ]; then
            RemoveDriver
            echo "(E) No X input modules directory found."
            echo ""
            exit 1
        fi
    fi

    mouse=${mousepath##*/}
    if [ ${mouse} = "vmmouse_drv.so" ]; then
        xmodule="irtouch_drv.so"
        CheckXorgVersion
        xpath=${mousepath%/*}
    elif [ ${mouse} = "vmmouse_drv.o" ]; then
        xmodule="irtouch_drv.o"
        xdirectory="x67"
        xpath=${xModulesPath}
        echo "(I) Check X window version: 6.7 ~ 6.8.2"
    else
        RemoveDriver
        echo "(E) Unknown mouse module!"
        echo ""
        exit 1
    fi

    if [ -e ${xpath}/${xmodule} ]; then
        rm -f ${xpath}/${xmodule}
    fi
    echo "(I) Copy X module: ${xdirectory}/${xmodule} to ${xpath}."
    cp -f ${installPath}/${companyName}/Module/${xdirectory}/${xmodule} ${xpath}
}

RemoveXorgModule() {
    xmodule="irtouch_drv.*o"
    if [ -e ${xModulesPath}/${xmodule} ]; then
        rm -f ${xModulesPath}/${xmodule}
    elif [ -e ${xModulesPath2}/${xmodule} ]; then
        rm -f ${xModulesPath2}/${xmodule}
    else
        echo "(I) Searching the X module of touch device."
        fullmodulepath=`find /usr -name ${xmodule}`
        if [ -z "${fullmodulepath}" ]; then
            RemoveDriver
            echo "(E) No X module of touch device found."
            echo ""
            exit 1
        elif [ -e ${fullmodulepath} ]; then
            rm -f ${fullmodulepath}
        fi
    fi
    echo "(I) Removed X module."
}

Add2Blacklist() {
    if [ -w ${blackListPath} ]; then
        grep -q $1 ${blackListPath}
        if [ $? -eq 1 ]; then
            filelines=`cat ${blackListPath} | wc -l`
            echo "(I) Add kernel module $1 into ${blackListPath}."
            sed -i ''${filelines}'a\# added by IRTOUCH installer\nblacklist '$1'' ${blackListPath}
        else
            echo "(I) The kernel module $1 has been added in ${blackListPath}."
        fi
    elif [ -w ${blackListPath1} ]; then
        grep -q $1 ${blackListPath1}
        if [ $? -eq 1 ]; then
            filelines=`cat ${blackListPath1} | wc -l`
            echo "(I) Add kernel module $1 into ${blackListPath1}."
            sed -i ''${filelines}'a\# added by IRTOUCH installer\nblacklist '$1'' ${blackListPath1}
        else
            echo "(I) The kernel module $1 has been added in ${blackListPath1}."
        fi
    elif [ -w ${blackListPath2} ]; then
        grep -q $1 ${blackListPath2}
        if [ $? -eq 1 ]; then
            filelines=`cat ${blackListPath2} | wc -l`
            echo "(I) Add kernel module $1 into ${blackListPath2}."
            sed -i ''${filelines}'a\# added by IRTOUCH installer\nblacklist '$1'' ${blackListPath2}
        else
            echo "(I) The kernel module $1 has been added in ${blackListPath2}."
        fi
    elif [ -w ${blackListPath3} ]; then
        grep -q $1 ${blackListPath3}
        if [ $? -eq 1 ]; then
            filelines=`cat ${blackListPath3} | wc -l`
            echo "(I) Add kernel module $1 into ${blackListPath3}."
            sed -i ''${filelines}'a\# added by IRTOUCH installer\n'$1'' ${blackListPath3}
        else
            echo "(I) The kernel module $1 has been added in ${blackListPath3}."
        fi
    else
        echo "(I) No blacklist file found in /etc/modprobe.d or /etc/hotplug."
    fi
}

CheckModuleAndBlacklist() {
    checkmod="lsmod"
    modfile="mod.info"
    ${checkmod} > ${modfile} 2> ${trash}
    grep -q "usbtouchscreen" ${modfile}
    #if [ $? -eq 0 ]; then
        echo "(I) Found inbuilt kernel module: usbtouchscreen."
		echo "(I) It is highly recommended that add it into blacklist."
        Add2Blacklist "usbtouchscreen"
    #fi
    rm -f ${modfile}
}

AddConfiguration() {
    echo "(I) Add touch configuration into $1."
    filelines=`cat $1 | wc -l`
    sed -i ''${filelines}'a\### Touch Configuration Beginning ###\
Section "InputDevice"\
        Identifier "'${companyName}'"\
        Driver "irtouch"\
        Option "ScrenNumber" "0"\
	Option "SendCoreEvents"\
	Option "MinX" "0"\
	Option "MinY" "0"\
	Option "MaxX" "4095"\
	Option "MaxY" "4095"\
        Option "SwapXY" "0"\
EndSection\
### Touch Configuration End ###' $1

    grep -q "Section \"ServerLayout\"" $1
    if [ $? -eq 0 ]; then
        sed -i '/Section "ServerLayout"/a\        InputDevice "'${companyName}'" "SendCoreEvents"' $1
    else
        echo "(W) No \"ServerLayout\" section found! It will be appended automatically."
        sed -i '/### Touch Configuration Beginning ###/a\
Section "ServerLayout"\
        Identifier "Default Layout"\
        InputDevice "'${companyName}'" "SendCoreEvents"\
EndSection\
' $1
        screenfile="screen.info"
        sed '/Section "Screen"/,/EndSection/!d' $1 > ${screenfile} 2>&1
        sed -i '/Identifier/!d' ${screenfile} 2>&1
        screen=`sed 's/Identifier/Screen/g' ${screenfile}`
        if [ -n "${screen}" ]; then
            sed -i '/Identifier "Default Layout"/a\'"${screen}"'' $1
        fi
        rm -f ${screenfile}
    fi

    checkx="X -version"
    xfile="X.info"
    ${checkx} > ${xfile} 2>&1
    grep -q "X.Org X Server 1.4.0.90" ${xfile}
    if [ $? -eq 0 ]; then
        grep -q "Build Operating System: Linux Ubuntu" ${xfile} # Add AutoMapping except Ubuntu 8.04
        if [ $? -eq 1 ]; then
            sed -i '/Option "ScreenNo"/a\        Option "AutoMapping" "1"' $1
        fi
    fi

    usbfile="usb.info"
    checkusb="lsusb -v -d 0eef:0001"
    ${checkusb} > ${usbfile} 2> ${trash}
    grep -q "Current Operating System: Linux asus" ${xfile} # for EPC series
    if [ $? -eq 0 ]; then
        grep -q "Human Interface Device" ${usbfile}
        if [ $? -eq 0 ]; then
            sed -i '/Option "ScreenNo"/a\        Option "HidOnEPC" "1"' $1
        else
            grep -q "Vendor Specific Protocol" ${usbfile}
            if [ $? -eq 1 ]; then
                echo "(I) Note that the option \"HidOnEPC\" \"1\" should be appended in the touch"
                echo "    section of X configuration file for HID compliant touch controller."
            fi
        fi
    else
        grep -q "ubuntu" ${xfile} # for Ubuntu series
        if [ $? -eq 0 ]; then
            grep -q "Human Interface Device" ${usbfile}
            if [ $? -eq 0 ]; then
                sed -i '/Option "ScreenNo"/a\        Option "SkipClick" "1"' $1
            fi
        fi
    fi
    rm -f ${usbfile}
    rm -f ${xfile}
}

RemoveXConfiguration() {
    sed -i '/"'${companyName}'" "SendCoreEvents"/d' $1
    sed -i '/### Touch Configuration Beginning ###/,/### Touch Configuration End ###/d' $1
    echo "(I) Removed touch configuration from $1."
}

EditConfigFile() {
    if [ -w ${xorgPath} ]; then
        echo "(I) Found X configuration file: ${xorgPath}."
        grep -q "### Touch Configuration Beginning ###" ${xorgPath}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xorgPath}
        else
            RemoveXConfiguration ${xorgPath}
            AddConfiguration ${xorgPath}
        fi
    elif [ -w ${xConfigPath} ]; then
        echo "(I) Found X configuration file: ${xConfigPath}."
        grep -q "### Touch Configuration Beginning ###" ${xConfigPath}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xConfigPath}
        else
            RemoveXConfiguration ${xConfigPath}
            AddConfiguration ${xConfigPath}
        fi
    elif [ -w ${xConfigPath2} ]; then
        echo "(I) Found X configuration file: ${xConfigPath2}."
        grep -q "### Touch Configuration Beginning ###" ${xConfigPath2}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xConfigPath2}
        else
            RemoveXConfiguration ${xConfigPath2}
            AddConfiguration ${xConfigPath2}
        fi
    else
       # RemoveDriver
       # RemoveXorgModule
       # echo "(E) No X configuration file found."
       # echo ""
       # exit 1
	cp -f ${installPath}/${companyName}/xorg.conf /etc/X11/
    fi
}

RestoreXConfigFile() {
    if [ -w ${xorgPath} ]; then
        RemoveXConfiguration ${xorgPath}
    elif [ -w ${xConfigPath} ]; then
        RemoveXConfiguration ${xConfigPath}
    elif [ -w ${xConfigPath2} ]; then
        RemoveXConfiguration ${xConfigPath2}
    else
        echo "(E) No X configuration file found."
        echo ""
        exit 1
    fi
}

EditLocalFile() {
    if [ -w ${localPath} ]; then
        grep -q $servicePath ${localPath}
        if [ $? -eq 1 ]; then
             filelines=`cat ${localPath} | wc -l`
             echo "(I) Add irtouch Service $1 into ${localPath}."
             sed -i ''${filelines}'a\# added by IRTOUCH installer\n'$servicePath' start &' ${localPath}
        else
             echo "(I) The $servicePath has been added in ${localPath}."
        fi
    elif [ -w ${localPath1} ]; then
        grep -q $servicePath ${localPath1}
         if [ $? -eq 1 ]; then
             filelines=`cat ${localPath1} | wc -l`
             echo "(I) Add irtouch Service $1 into ${localPath1}."
             sed -i ''${filelines}'a\# added by IRTOUCH installer\n'$servicePath' start &' ${localPath1}
        else
             echo "(I) The $servicePath has been added in ${localPath1}."
        fi
    elif [ -w ${localPath2} ]; then
        grep -q $servicePath ${localPath2}
         if [ $? -eq 1 ]; then
             filelines=`cat ${localPath2} | wc -l`
             echo "(I) Add irtouch Service $1 into ${localPath2}."
             sed -i ''${filelines}'a\# added by IRTOUCH installer\n'$servicePath' start &' ${localPath2}
        else
             echo "(I) The $servicePath has been added in ${localPath2}."
        fi
    elif [ -w ${localPath3} ]; then
        grep -q $servicePath ${localPath3}
         if [ $? -eq 1 ]; then
             filelines=`cat ${localPath3} | wc -l`
             echo "(I) Add irtouch Service $1 into ${localPath3}."
             sed -i ''${filelines}'a\# added by IRTOUCH installer\n'$servicePath' start &' ${localPath3}
        else
             echo "(I) The $servicePath has been added in ${localPath3}."
        fi
    else
		#echo "(E) No rc.local or boot.local file found."
		cp ${installPath}/${companyName}/rc.local ${localPath}
    fi
}

RemoveLocalFile() {
    if [ -w ${localPath} ]; then
        grep -q $servicePath ${localPath}
        if [ $? -eq 0 ]; then
             sed -i '/# added by IRTOUCH installer/,/start/d' ${localPath}
             echo "(I) Removed $servicePath from ${localPath}."
        fi
    elif [ -w ${localPath1} ]; then
        grep -q $servicePath ${localPath1}
         if [ $? -eq 0 ]; then
             sed -i '/# added by IRTOUCH installer/,/start/d' ${localPath1}
             echo "(I) Remove $servicePath from ${localPath1}."
        fi
    elif [ -w ${localPath2} ]; then
        grep -q $servicePath ${localPath2}
         if [ $? -eq 0 ]; then
             sed -i '/# added by IRTOUCH installer/,/start/d' ${localPath2}
             echo "(I) Remove $servicePath from ${localPath2}."
        fi
    elif [ -w ${localPath3} ]; then
        grep -q $servicePath ${localPath3}
         if [ $? -eq 0 ]; then
             sed -i '/# added by IRTOUCH installer/,/start/d' ${localPath3}
             echo "(I) Remove $servicePath from ${localPath3}."
        fi
    else
	echo "(E) No rc.local or boot.local file found."
    fi
}

RemoveConfigFile() {
    if [ -e ${saveConfigPath}/${tsConfigName} ]; then
	rm -rf ${saveConfigPath}/${companyName}
    	echo "(I) Remove configuration file from /etc."
    else
       	echo "(I) No configuration file from /etc found."
    fi
}

clear
ShowTitle

if [ $# = 0 ]; then
    CheckPermission
    CheckModuleAndBlacklist
    CheckOldDriver
    ExtractDriver
    EditLocalFile
    CopyXorgMudule
    EditConfigFile
    echo ""
    echo "(I) Please reboot the system."
elif [ $# = 1 ]; then
    if [ $1 = "uninstall" ]; then
        CheckPermission
        echo "(I) Begin to remove IRTOUCH driver."
        RemoveDriver
	RemoveConfigFile
        RemoveXorgModule
        RestoreXConfigFile
	RemoveLocalFile
        echo ""
        echo "(I) The IRTOUCH Touch Screen driver has been removed successfully."
        echo "(I) Please reboot the system."
    elif [ $1 = "version" ]; then
        echo "(I) Version: ${version}"
    else
        echo "(I) Usage: [sh $0] to install IRTOUCH Touch Screen driver."
	echo "           [sh $0 uninstall] to uninstall IRTOUCH Touch Screen driver."
    fi
else
    echo "(I) Usage: [sh $0] to install IRTOUCH Touch Screen driver."
    echo "           [sh $0 uninstall] to uninstall IRTOUCH Touch Screen driver."
fi

echo ""
